﻿using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Text;

namespace TestProject
{
    /// <summary>
    /// 아쿠아 버튼 그리기 객체
    /// </summary>
    public class AquaButtonDrawingObject : BaseDrawingObject, IDrawingObject
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 오른쪽 그리기 객체
        /// </summary>
        public AquaButtonRightDrawingObject RightDrawingObject = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 명칭 - Name

        /// <summary>
        /// 명칭
        /// </summary>
        public string Name
        {
            get
            {
                return "AquaButton";
            }
        }

        #endregion
        #region 자식 그리기 객체 배열 - ChildArray

        /// <summary>
        /// 자식 그리기 객체 배열
        /// </summary>
        public override IDrawingObject[] ChildArray
        {
            get
            {
                IDrawingObject[] drwaingObjectArray = new IDrawingObject[1];

                drwaingObjectArray[0] = RightDrawingObject;

                return drwaingObjectArray;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - AquaButtonDrawingObject()

        /// <summary>
        /// 생성자
        /// </summary>
        public AquaButtonDrawingObject()
        {
            this.parent = null;

            this.isVisible = true;

            this.cacheGraphicsPath = new GraphicsPath();

            RightDrawingObject = new AquaButtonRightDrawingObject(this);

            this.hashTable = new Hashtable();

            foreach(IDrawingObject child in ChildArray)
            {
                this.hashTable.Add(child.Name.ToLower(), child);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 변환하기 - Transform(sourceMatrix)

        /// <summary>
        /// 변환하기
        /// </summary>
        /// <param name="sourceMatrix">소스 매트릭스</param>
        /// <returns>변환 매트릭스</returns>
        public override Matrix Transform(Matrix sourceMatrix)
        {
            Matrix targetMatrix = new Matrix();

            IDrawingObject currentDrawingObject = this;

            while(currentDrawingObject != null)
            {
                targetMatrix.Multiply(currentDrawingObject.GetRelativeTransform(), MatrixOrder.Append);

                currentDrawingObject = currentDrawingObject.Parent;
            }

            targetMatrix.Multiply(sourceMatrix, MatrixOrder.Append);

            return targetMatrix;
        }

        #endregion

        #region 칠하기 - Paint(graphics, sourceMatrix)

        /// <summary>
        /// 칠하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="sourceMatrix">소스 매트릭스</param>
        public override void Paint(Graphics graphics, Matrix sourceMatrix)
        {
            if(!IsVisible)
            {
                return;
            }
                
            GraphicsPath graphicsPath       = new GraphicsPath();
            GraphicsPath radialGraphicsPath    = null;
            Pen          pen                = null;

            Matrix saveMatrix = graphics.Transform.Clone();
                
            if(this.SmoothingMode != SmoothingMode.Invalid)
            {
                graphics.SmoothingMode = this.SmoothingMode;
            }

            if(this.TextRenderingHint != TextRenderingHint.SystemDefault)
            {
                graphics.TextRenderingHint = this.TextRenderingHint;
            }

            graphics.Transform = this.Transform(sourceMatrix);

            RightDrawingObject.Paint(graphics, sourceMatrix);

            if(pen != null)
            {
                pen.Dispose();
            }

            if(radialGraphicsPath != null)
            {
                radialGraphicsPath.Dispose();
            }

            if(graphicsPath != null)
            {
                graphicsPath.Dispose();
            }

            graphics.Transform = saveMatrix;
        }

        #endregion
    }
}