﻿using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Text;

namespace TestProject
{
    /// <summary>
    /// 고양이/배경 그리기 객체
    /// </summary>
    public class CatBackgroundDrawingObject : BaseDrawingObject, IDrawingObject
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 명칭 - Name

        /// <summary>
        /// 명칭
        /// </summary>
        public string Name
        {
            get
            {
                return "Background";
            }
        }

        #endregion
        #region 자식 그리기 객체 배열 - ChildArray

        /// <summary>
        /// 자식 그리기 객체 배열
        /// </summary>
        public override IDrawingObject[] ChildArray
        {
            get
            {
                IDrawingObject[] drawingObjectArray = new IDrawingObject[0];

                return drawingObjectArray;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CatBackgroundDrawingObject(parent)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="parent">부모 그리기 객체</param>
        public CatBackgroundDrawingObject(IDrawingObject parent)
        {
            this.parent = parent;

            this.isVisible = true;

            this.cacheGraphicsPath = new GraphicsPath();
                
            this.cacheGraphicsPath.StartFigure();
            this.cacheGraphicsPath.AddLine(611.71484375F, 767F, -1F, 767F);
            this.cacheGraphicsPath.AddLine(-1F, -24.927734375F, 611.71484375F, -24.927734375F);
            this.cacheGraphicsPath.AddLine(611.71484375F, -24.927734375F, 611.71484375F, 767F);
            this.cacheGraphicsPath.CloseFigure();

            this.hashTable = new Hashtable();

            foreach(IDrawingObject child in ChildArray)
            {
                this.hashTable.Add(child.Name.ToLower(), child);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 변환하기 - Transform(sourceMatrix)

        /// <summary>
        /// 변환하기
        /// </summary>
        /// <param name="sourceMatrix">소스 매트릭스</param>
        /// <returns>변환 매트릭스</returns>
        public override Matrix Transform(Matrix sourceMatrix)
        {
            Matrix targetMatrix = new Matrix();

            IDrawingObject currentDrawingObject = this;

            while(currentDrawingObject != null)
            {
                targetMatrix.Multiply(currentDrawingObject.GetRelativeTransform(), MatrixOrder.Append);

                currentDrawingObject = currentDrawingObject.Parent;
            }

            targetMatrix.Multiply(sourceMatrix, MatrixOrder.Append);

            return targetMatrix;
        }

        #endregion

        #region 칠하기 - Paint(graphics, sourceMatrix)

        /// <summary>
        /// 칠하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="sourceMatrix">소스 매트릭스</param>
        public override void Paint(Graphics graphics, Matrix sourceMatrix)
        {
            if(!IsVisible)
            {
                return;
            }
                
            GraphicsPath graphicsPath       = new GraphicsPath();
            GraphicsPath radialGraphicsPath = null;
            Pen          pen                = null;
            Brush        brush              = null;

            Matrix saveMatrix = graphics.Transform.Clone();
                
            if(this.SmoothingMode != SmoothingMode.Invalid)
            {
                graphics.SmoothingMode = this.SmoothingMode;
            }

            if(this.TextRenderingHint != TextRenderingHint.SystemDefault)
            {
                graphics.TextRenderingHint = this.TextRenderingHint;
            }

            graphics.Transform = this.Transform(sourceMatrix);
                
                
            graphicsPath.StartFigure();
            graphicsPath.AddLine(611.71484375F, 767F, -1F, 767F);
            graphicsPath.AddLine(-1F, -24.927734375F, 611.71484375F, -24.927734375F);
            graphicsPath.AddLine(611.71484375F, -24.927734375F, 611.71484375F, 767F);
            graphicsPath.CloseFigure();
                
            brush = new SolidBrush(Color.FromArgb(255, 10, 7, 24));

            graphicsPath.FillMode = FillMode.Winding;

            graphics.FillPath(brush, graphicsPath);

            if(brush != null) brush.Dispose();
                
            graphicsPath.Reset();

            if(pen != null)
            {
                pen.Dispose();
            }

            if(radialGraphicsPath != null)
            {
                radialGraphicsPath.Dispose();
            }

            if(graphicsPath != null)
            {
                graphicsPath.Dispose();
            }

            graphics.Transform = saveMatrix;
        }

        #endregion
    }
}