﻿using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// 영역 헬퍼
    /// </summary>
    public class RegionHelper : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Import
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 영역 결합하기 - CombineRgn(targetRegionHandle, sourceRegionHandle1, sourceRegionHandle2, combineMode)

        /// <summary>
        /// 영역 결합하기
        /// </summary>
        /// <param name="targetRegionHandle">타겟 영역 핸들</param>
        /// <param name="sourceRegionHandle1">소스 영역 핸들 1</param>
        /// <param name="sourceRegionHandle2">소스 영역 핸들 2</param>
        /// <param name="combineMode">결합 모드</param>
        /// <returns>처리 결과</returns>
		[DllImport("gdi32")] public	static extern int CombineRgn(IntPtr targetRegionHandle, IntPtr sourceRegionHandle1, IntPtr sourceRegionHandle2, int combineMode);

        #endregion
        #region 객체 삭제하기 - DeleteObject(objectHandle)

        /// <summary>
        /// 객체 삭제하기
        /// </summary>
        /// <param name="objectHandle">객체 핸들</param>
        /// <returns>처리 결과</returns>
		[DllImport("gdi32")] public	static extern int DeleteObject(IntPtr objectHandle);

        #endregion
        #region 영역 데이터 구하기 - GetRegionData(regionHandle, count, regionDataArray)

        /// <summary>
        /// 영역 데이터 구하기
        /// </summary>
        /// <param name="regionHandle">영역 핸들</param>
        /// <param name="count">카운트</param>
        /// <param name="regionDataArray">영역 데이터 배열</param>
        /// <returns>처리 결과</returns>
		[DllImport("gdi32")] public	static extern int GetRegionData(IntPtr regionHandle, int count, [MarshalAs(UnmanagedType.LPArray)] int[] regionDataArray);

        #endregion
        #region 다수 다각형 영역 생성하기 - CreatePolyPolygonRgn(point, polyCount, count, polyFillMode)

        /// <summary>
        /// 다수 다각형 영역 생성하기
        /// </summary>
        /// <param name="point">포인트</param>
        /// <param name="polyCount">다수 카운트</param>
        /// <param name="count">카운트</param>
        /// <param name="polyFillMode">다수 칠하기 모드</param>
        /// <returns>처리 결과</returns>
		[DllImport("gdi32")] public	static extern int CreatePolyPolygonRgn(ref POINT point, ref int polyCount, int count, int polyFillMode);

        #endregion
        #region 영역 색성하기 - ExtCreateRegion(xFormArray, count, regionDataArray)

        /// <summary>
        /// 영역 색성하기
        /// </summary>
        /// <param name="xFormArray">X 폼 배열</param>
        /// <param name="count">카운트</param>
        /// <param name="regionDataArray">영역 데이터 배열</param>
        /// <returns>처리 결과</returns>
		[DllImport("gdi32")] public	static extern int ExtCreateRegion([MarshalAs(UnmanagedType.LPArray)] int[] xFormArray, int count, int[] regionDataArray);

        #endregion
        #region 영역 위치 여부 구하기 - PtInRegion(targetRegionHandle, x, y)

        /// <summary>
        /// 영역 위치 여부 구하기
        /// </summary>
        /// <param name="targetRegionHandle">타겟 영역 핸들</param>
        /// <param name="x">X 좌표</param>
        /// <param name="y">Y 좌표</param>
        /// <returns>처리 결과</returns>
		[DllImport("gdi32")] public	static extern int PtInRegion(IntPtr targetRegionHandle, int x, int y);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// ALTERNATE
        /// </summary>
		public const int ALTERNATE = 1;

        /// <summary>
        /// WINDING
        /// </summary>
		public const int WINDING = 2;

        /// <summary>
        /// NULLREGION
        /// </summary>
		public const int NULLREGION	= 1;

        /// <summary>
        /// RGN_AND
        /// </summary>
		public const int RGN_AND = 1;

        /// <summary>
        /// RGN_COPY
        /// </summary>
		public const int RGN_COPY =	5;

        /// <summary>
        /// RGN_DIFF
        /// </summary>
		public const int RGN_DIFF =	4;

        /// <summary>
        /// RGN_MAX
        /// </summary>
		public const int RGN_MAX = RGN_COPY;

        /// <summary>
        /// RGN_MIN
        /// </summary>
		public const int RGN_MIN = RGN_AND;

        /// <summary>
        /// RGN_OR
        /// </summary>
		public const int RGN_OR	= 2;

        /// <summary>
        /// RGN_XOR
        /// </summary>
		public const int RGN_XOR = 3;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 영역 핸들
        /// </summary>
        private IntPtr regionHandle = (IntPtr)0;

        /// <summary>
        /// 스캔 데이터 배열
        /// </summary>
		private	Rectangle[] scanDataArray = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RegionHelper()

        /// <summary>
        /// 생성자
        /// </summary>
        public RegionHelper()
        {
            SetEmptyRegion();
        }

        #endregion
        #region 생성자 - RegionHelper(graphicsPath)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="graphicsPath">그래픽스 경로</param>
        public RegionHelper(GraphicsPath graphicsPath)
        {
            SetRegion(graphicsPath);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Destructor

        #region 소멸자 - ~RegionHelper()

        /// <summary>
        /// 소멸자
        /// </summary>
        ~RegionHelper()
        {
            Dispose(false);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 리소스 해제하기 - Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        public void	Dispose()
		{
            Dispose(true);

            GC.SuppressFinalize(this);
		}

        #endregion
        #region 리소스 해제하기 - Dispose(disposing)

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        /// <param name="disposing">해제 여부</param>
        protected virtual void Dispose(bool disposing)
        {
            if(disposing)
            {
                this.scanDataArray = null;
            }

            DeleteRegion();
        }

        #endregion

        #region 포함 여부 구하기 - Contains(x, y)

        /// <summary>
        /// 포함 여부 구하기
        /// </summary>
        /// <param name="x">X 좌표</param>
        /// <param name="y">Y 좌표</param>
        /// <returns></returns>
        public bool Contains(int x, int y)
        {
            return PtInRegion(this.regionHandle, x, y) != 0;
        }

        #endregion
        #region 빈 영역 설정하기 - SetEmptyRegion()

        /// <summary>
        /// 빈 영역 설정하기
        /// </summary>
        public void	SetEmptyRegion()
        {
            this.scanDataArray = null;

            int[] dataArray = {32, 1, 0, 0, 0, 0, 0, 0, 0};

            DeleteRegion();

            this.regionHandle = (IntPtr)ExtCreateRegion(null, dataArray.Length * 4, dataArray);
        }

        #endregion
        #region 합치기 - Union(regionHelper)

        /// <summary>
        /// 합치기
        /// </summary>
        /// <param name="regionHelper">영역 헬퍼</param>
        public void Union(RegionHelper regionHelper)
        {
            CombineRgn(this.regionHandle, this.regionHandle, regionHelper.regionHandle, RGN_OR);

            this.scanDataArray = null;
        }

        #endregion
        #region 스캔 영역 배열 구하기 - GetScanRegionArray()

        /// <summary>
        /// 스캔 영역 배열 구하기
        /// </summary>
        /// <returns>스캔 영역 배열</returns>
        public Rectangle[] GetScanRegionArray()
        {
            if(this.scanDataArray == null)
            {
                this.scanDataArray = new Rectangle[0];

                if(this.regionHandle != (IntPtr)0)
                {
                    int regionDataSize = GetRegionData(this.regionHandle, 0, null);

                    int[] regionDataArray = new int[regionDataSize / 4];
  
                    GetRegionData(this.regionHandle, regionDataSize, regionDataArray);

                    int rectangleCount = regionDataArray[3] / 16;
  
                    this.scanDataArray = new Rectangle[rectangleCount];

                    for(int i = 0; i < rectangleCount; i++)
                    {
                        int left   = regionDataArray[8 + (i * 4)    ];
                        int top    = regionDataArray[8 + (i * 4) + 1];
                        int right  = regionDataArray[8 + (i * 4) + 2];
                        int bottom = regionDataArray[8 + (i * 4) + 3];

                        this.scanDataArray[i] = new Rectangle(left, top, right - left + 1, bottom - top + 1);
                    }
                }
            }

            return this.scanDataArray;
        }

        #endregion
        #region 영역 구하기 - GetRegion()

        /// <summary>
        /// 영역 구하기
        /// </summary>
        /// <returns>영역</returns>
        public Region GetRegion()
        {
            Region region = new Region();

            region.MakeEmpty();

            Rectangle[] rectangleArray = GetScanRegionArray();

            foreach(Rectangle rectangle in rectangleArray)
            {
                region.Union(rectangle);
            }

            return region;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 영역 삭제하기 - DeleteRegion()

        /// <summary>
        /// 영역 삭제하기
        /// </summary>
        private void DeleteRegion()
        {
            this.scanDataArray = null;

            if(this.regionHandle != (IntPtr)0)
            {
                DeleteObject(this.regionHandle);

                this.regionHandle = (IntPtr)0;
            }
        }

        #endregion
        #region 영역 설정하기 - SetRegion(graphicsPath)

        /// <summary>
        /// 영역 설정하기
        /// </summary>
        /// <param name="graphicsPath">그래픽스 객체</param>
        private void SetRegion(GraphicsPath graphicsPath)
        {
            this.scanDataArray = null;

            if(graphicsPath != null && graphicsPath.PointCount != 0)
            {
                ArrayList    targetArrayList    = new ArrayList();
                GraphicsPath targetGraphicsPath = graphicsPath.Clone() as GraphicsPath;

                PointF[] temporaryPointArray;
                byte[]   targetPathTypeArray;
                POINT[]  targetPointArray;
                int      count = 0;
  
                targetGraphicsPath.Flatten(null, .1f);

                temporaryPointArray = targetGraphicsPath.PathPoints;
                targetPathTypeArray = targetGraphicsPath.PathTypes;
                targetPointArray    = new POINT[temporaryPointArray.Length];
  
                for(int i = 0; i < temporaryPointArray.Length; i++)
                {
                    targetPointArray[i].X = (int) temporaryPointArray[i].X;
                    targetPointArray[i].Y = (int) temporaryPointArray[i].Y;

                    if(((PathPointType)targetPathTypeArray[i]) == PathPointType.Start)
                    {
                        if(count > 0)
                        {
                            targetArrayList.Add(count);
                        }

                        count = 0;
                    }
            
                    count++;
                }
  
                if(count > 0)
                {
                    targetArrayList.Add(count);
                }

                int[] countArray = (int[])targetArrayList.ToArray(typeof(int));

                if(countArray != null && countArray.Length > 0 && targetPointArray != null && targetPointArray.Length > 0)
                {
                    DeleteRegion();

                    switch(targetGraphicsPath.FillMode)
                    {
                        case FillMode.Alternate :

                            this.regionHandle = (IntPtr)CreatePolyPolygonRgn(ref targetPointArray[0], ref countArray[0], targetArrayList.Count, ALTERNATE);

                            break;

                        case FillMode.Winding :

                            this.regionHandle = (IntPtr) CreatePolyPolygonRgn(ref targetPointArray[0], ref countArray[0], targetArrayList.Count, WINDING);

                            break;
                    }
                }

                targetGraphicsPath.Dispose();
            }
            else
            {
                SetEmptyRegion();
            }
        }

        #endregion
    }
}