﻿using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Drawing.Text;

namespace TestProject
{
    /// <summary>
    /// 그리기 객체 인터페이스
    /// </summary>
	public interface IDrawingObject
	{
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property

        #region 명칭 - Name

        /// <summary>
        /// 명칭
        /// </summary>
        string Name { get; }

        #endregion

        #region 부모 그리기 객체 - Parent

        /// <summary>
        /// 부모 그리기 객체
        /// </summary>
		IDrawingObject Parent { get; }

        #endregion
        #region 자식 그리기 객체 배열 - ChildArray

        /// <summary>
        /// 자식 그리기 객체 배열
        /// </summary>
        IDrawingObject[] ChildArray { get; }

        #endregion

        #region 스무딩 모드 - SmoothingMode

        /// <summary>
        /// 스무딩 모드
        /// </summary>
        SmoothingMode SmoothingMode { get; set; }

        #endregion
        #region 텍스트 렌더링 힌트 - TextRenderingHint

        /// <summary>
        /// 텍스트 렌더링 힌트
        /// </summary>
        TextRenderingHint TextRenderingHint { get; set; }

        #endregion
        #region 표시 여부 - IsVisible

        /// <summary>
        /// 표시 여부
        /// </summary>
        bool IsVisible { get; set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method

        #region 자식 찾기 - FindChild(name)

        /// <summary>
        /// 자식 찾기
        /// </summary>
        /// <param name="name">명칭</param>
        /// <returns>자식 그리기 객체</returns>
		IDrawingObject FindChild(string name);

        #endregion

        #region 변환하기 - Transform()

        /// <summary>
        /// 변환하기
        /// </summary>
        /// <returns>변환 매트릭스</returns>
        Matrix Transform();

        #endregion
        #region 변환하기 - Transform(sourceMatrix)

        /// <summary>
        /// 변환하기
        /// </summary>
        /// <param name="sourceMatrix">소스 매트릭스</param>
        /// <returns>변환 매트릭스</returns>
        Matrix Transform(Matrix sourceMatrix);

        #endregion

        #region 상대적 변환 설정하기 - SetRelativeTransform(sourceMatrix)

        /// <summary>
        /// 상대적 변환 설정하기
        /// </summary>
        /// <param name="sourceMatrix">소스 매트릭스</param>
        void SetRelativeTransform(Matrix sourceMatrix);

        #endregion
        #region 상대적 변환 구하기 - GetRelativeTransform()

        /// <summary>
        /// 상대적 변환 구하기
        /// </summary>
        /// <returns>매트릭스</returns>
        Matrix GetRelativeTransform();

        #endregion
        #region 변환 리셋하기 - ResetTransforms()

        /// <summary>
        /// 변환 리셋하기
        /// </summary>
        void ResetTransforms();

        #endregion
        #region 표준 변환 구하기 - GetStandardTransform(sourceRectangle)

        /// <summary>
        /// 표준 변환 구하기
        /// </summary>
        /// <param name="sourceRectangle">소스 사각형</param>
        /// <returns>표준 변환 매트릭스</returns>
        Matrix GetStandardTransform(RectangleF sourceRectangle);

        #endregion

        #region 그래픽스 경로 구하기 - GetGraphicsPath()

        /// <summary>
        /// 그래픽스 경로 구하기
        /// </summary>
        /// <returns>그래픽스 경로</returns>
        GraphicsPath GetGraphicsPath();

        #endregion
        #region 그래픽스 경로 구하기 - GetGraphicsPath(sourceMatrix)

        /// <summary>
        /// 그래픽스 경로 구하기
        /// </summary>
        /// <param name="sourceMatrix">소스 매트릭스</param>
        /// <returns>그래픽스 경로</returns>
        GraphicsPath GetGraphicsPath(Matrix sourceMatrix);

        #endregion

        #region 영역 구하기 - GetRegion()

        /// <summary>
        /// 영역 구하기
        /// </summary>
        /// <returns>영역</returns>
        Region GetRegion();

        #endregion
        #region 영역 구하기 - GetRegion(sourceMatrix)

        /// <summary>
        /// 영역 구하기
        /// </summary>
        /// <param name="sourceMatrix">소스 매트릭스</param>
        /// <returns>영역</returns>
        Region GetRegion(Matrix sourceMatrix);

        #endregion

        #region 바운딩 사각형 구하기 - GetBoundingRectangle()

        /// <summary>
        /// 바운딩 사각형 구하기
        /// </summary>
        /// <returns>바운딩 사각형</returns>
        RectangleF GetBoundingRectangle();

        #endregion
        #region 바운딩 사각형 구하기 - GetBoundingRectangle(sourceMatrix)

        /// <summary>
        /// 바운딩 사각형 구하기
        /// </summary>
        /// <param name="sourceMatrix">소스 매트릭스</param>
        /// <returns>바운딩 사각형</returns>
        RectangleF GetBoundingRectangle(Matrix sourceMatrix);

        #endregion

        #region 중심점 구하기 - GetCenterPoint()

        /// <summary>
        /// 중심점 구하기
        /// </summary>
        /// <returns>중심점</returns>
        PointF GetCenterPoint();

        #endregion
        #region 중심점 구하기 - GetCenterPoint(sourceMatrix)

        /// <summary>
        /// 중심점 구하기
        /// </summary>
        /// <param name="sourceMatrix">소스 매트릭스</param>
        /// <returns>중심점</returns>
        PointF GetCenterPoint(Matrix sourceMatrix);

        #endregion

        #region 칠하기 - Paint(graphics)

        /// <summary>
        /// 칠하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        void Paint(Graphics graphics);

        #endregion
        #region 칠하기 - Paint(graphics, sourceMatrix)

        /// <summary>
        /// 칠하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="sourceMatrix">소스 매트릭스</param>
        void Paint(Graphics graphics, Matrix sourceMatrix);

        #endregion
        #region 칠하기 - Paint(graphics, boundRectangle)

        /// <summary>
        /// 칠하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="boundRectangle">테두리 사각형</param>
        void Paint(Graphics graphics, RectangleF boundRectangle);

        #endregion
        #region 칠하기 - Paint(graphics, boundRectangle, quality)

        /// <summary>
        /// 칠하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="boundRectangle">테두리 사각형</param>
        /// <param name="quality">품질</param>
        void Paint(Graphics graphics, RectangleF boundRectangle, float quality);

        #endregion
        #region 칠하기 - Paint(graphics, boundRectangle, quality, targetMatrix)

        /// <summary>
        /// 칠하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="boundRectangle">테두리 사각형</param>
        /// <param name="quality">품질</param>
        /// <param name="targetMatrix">타겟 매트릭스</param>
        void Paint(Graphics graphics, RectangleF boundRectangle, float quality, out Matrix targetMatrix);

        #endregion
        #region 칠하기 - Paint(graphics, boundRectangle, quality, targetMatrix, imageAttribute)

        /// <summary>
        /// 칠하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="boundRectangle">테두리 사각형</param>
        /// <param name="quality">품질</param>
        /// <param name="targetMatrix">타겟 매트릭스</param>
        /// <param name="imageAttribute">이미지 어트리뷰트</param>
        void Paint(Graphics graphics, RectangleF boundRectangle, float quality, out Matrix targetMatrix, ImageAttributes imageAttribute);

        #endregion
	}
}